package net.sf.corn.exception;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * This class is written to simplify exception catching code routines and helping
 * to write more dry codes. It supports {@link ExceptionCommand} executions and 
 * notification  of {@link ExceptionListener} implemented listeners.Thus, you can reuse 
 * predefined commands and listeners again and again.
 * <p><code>
 * ExceptionCatcher.trap(exception).execute(logEx).notify(eventListener).process();
 * </code>
 * Although it's possible to attach commands and listeners to each exception catching
 * process, you can also define default commands and common listeners that will be notified 
 * in each exception catching process. 
 * </p><code>
 * ExceptionCatcher.addCommonListener(exceptionListener);
 * ExceptionCatcher.addDefaultCommand(command1);
 * </code>
 * 
 * @author Serhat Dirik
 * 
 */
public class ExceptionCatcher {

	/* STATIC PART */

	/**
	 * List of the default commands to be executed when exception get caught
	 */
	private static  List<ExceptionCommand> defaultCommands = new CopyOnWriteArrayList<>();
	/**
	 * List of the default listener to be triggered when exception get caught
	 */
	private static  List<ExceptionListener> commonListeners = new CopyOnWriteArrayList<>();

	/**
	 * Gives the list of the default commands to be executed for each exception
	 * 
	 * @return list of the default commands
	 */
	public static List<ExceptionCommand> getDefaultCommands() {
		return Collections.unmodifiableList(defaultCommands);
	}

	/**
	 * Tells how many default command exist
	 * 
	 * @return number of the default commands
	 */
	public static int sizeOfDefaultCommands() {
		return defaultCommands.size();
	}

	/**
	 * Tells if the given command in the defaults list
	 * 
	 * @param command
	 *            the command to be check if its in the list
	 * @return true if the command in the defaults list
	 */
	public static boolean containsDefaultCommand(ExceptionCommand command) {
		return defaultCommands.contains(command);
	}

	/**
	 * Adds a command to the defaults list, so it is executed each exception
	 * 
	 * @param command
	 *            The command to be added in the list
	 * @return true if successfully added
	 */
	public static boolean addDefaultCommand(ExceptionCommand command) {
		return defaultCommands.add(command);
	}

	/**
	 * Removes a command from the defaults
	 * 
	 * @param command
	 *            the command to be removed from the list
	 * @return true if successfully removed from the defaults list
	 */
	public static boolean removeDefaultCommand(ExceptionCommand command) {
		return defaultCommands.remove(command);
	}

	/**
	 * Removes whole default commands
	 */
	public static void clearDefaultCommands() {
		defaultCommands.clear();
	}

	/**
	 * Gives the list of common exception listeners
	 * 
	 * @return unmodifiable list of common exception listeners
	 */
	public static List<ExceptionListener> getCommonListeners() {
		return Collections.unmodifiableList(commonListeners);
	}

	/**
	 * Tells how many common listener are exist
	 * 
	 * @return number of the common listeners
	 */
	public static int sizeOfCommonListeners() {
		return commonListeners.size();
	}

	/**
	 * Tells whether the given listener exist in the common listeners list or
	 * not
	 * 
	 * @param listener
	 *            the listener to be checked
	 * @return true if the given listener is defined as a common listener
	 */
	public static boolean containsCommonListener(ExceptionListener listener) {
		return commonListeners.contains(listener);
	}

	/**
	 * Puts a listener in the common listeners list
	 * 
	 * @param listener
	 *            the listener to be added in the list of common listeners
	 * @return true if succeffully added
	 */
	public static boolean addCommonListener(ExceptionListener listener) {
		return commonListeners.add(listener);
	}

	/**
	 * Removes the given listener form the common listeners list
	 * 
	 * @param listener
	 *            the listener to be removed from the commons
	 * @return true if it successfully removed
	 */
	public static boolean removeCommonListener(ExceptionListener listener) {
		return commonListeners.remove(listener);
	}

	/**
	 * Clears the whole commons list
	 */
	public static void clearCommonListeners() {
		commonListeners.clear();
	}
	
	/**
	 * Creates a new ExceptionCatcher
	 * @param e Exception
	 * @return new ExceptionCather Object
	 */
	public static ExceptionCatcher trap(Exception e){
		return new ExceptionCatcher(e);
	}
	/* OBJECT PART */

	/**
	 * List of commands to be executed before build
	 */
	private  List<ExceptionCommand> commands = new ArrayList<>();
	/**
	 * List of the listeners that to be notified
	 */
	private  List<ExceptionListener> listeners = new ArrayList<>();
	
	private Exception exception = null;
 

	/* Constructors */
 
	public ExceptionCatcher(Exception ex) {
		this.exception = ex;
	}
 

	/* Object Methods */
	@Override
	public String toString() {
		return exception.toString();
	}

	/*
	 * Builder methods
	 */

	/**
	 * Adds a command to the commands stack which will be executed in order
	 * 
	 * @param command
	 *            command to be added to commands stack
	 * @return this
	 */
	public ExceptionCatcher execute(ExceptionCommand command) {
		commands.add(command);
		return this;
	}

	/**
	 * Adds the given listener to the listeners stack which will be notified
	 * 
	 * @param listener
	 * @return
	 */
	public ExceptionCatcher notify(ExceptionListener listener) {
		listeners.add(listener);
		return this;
	}
 

	/**
	 * Notifies listeners (private
	 * and common listener) and executes commands (private and default
	 * commands). Use this as the last command of your commands chain
	 * 
	 */
	public void process() {
		// Notify private listeners first
		for (ExceptionListener listener : listeners) {
			if (listener.isSubject(exception.getClass(),false)) {
				ExceptionEventsBus.execute(listener.action(exception,false));
			}
		}
		// Notify common listeners
		for (ExceptionListener listener : getCommonListeners()) {
			if (listener.isSubject(exception.getClass(),false)) {
				ExceptionEventsBus.execute(listener.action(exception,false));
			}
		}
		// Execute private commands
		for (ExceptionCommand command : commands) {
			try {
				command.execute(exception);
			} catch (Exception ex) {
			}
		}
		// Execute default commands
		for (ExceptionCommand command : getDefaultCommands()) {
			try {
				command.execute(exception);
			} catch (Exception ex) {
			}
		}
	}
}
